"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseInvitationUrl = void 0;
const query_string_1 = require("query-string");
const error_1 = require("../error");
const connections_1 = require("../modules/connections");
const helpers_1 = require("../modules/oob/helpers");
const messages_1 = require("../modules/oob/messages");
/**
 * Parses URL containing encoded invitation and returns invitation message.
 *
 * @param invitationUrl URL containing encoded invitation
 *
 * @returns OutOfBandInvitation
 */
const parseInvitationUrl = (invitationUrl) => {
    const parsedUrl = query_string_1.parseUrl(invitationUrl).query;
    if (parsedUrl['oob']) {
        const outOfBandInvitation = messages_1.OutOfBandInvitation.fromUrl(invitationUrl);
        return outOfBandInvitation;
    }
    else if (parsedUrl['c_i'] || parsedUrl['d_m']) {
        const invitation = connections_1.ConnectionInvitationMessage.fromUrl(invitationUrl);
        return helpers_1.convertToNewInvitation(invitation);
    }
    throw new error_1.AriesFrameworkError('InvitationUrl is invalid. It needs to contain one, and only one, of the following parameters: `oob`, `c_i` or `d_m`.');
};
exports.parseInvitationUrl = parseInvitationUrl;
//# sourceMappingURL=parseInvitation.js.map